#
# Copyright 2018 Fairphone B.V.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

# Disable the long line error caused by URL's
# pylint: disable=line-too-long, C0301
"""Android Maestro, an orchestrator of Android devices system load.

Overview
--------

The tool is architected around the following classes:

-   The :py:class:`shell.MaestroShell` is the main entry-point that provides an
    interactive shell to start and monitor system load sessions.
-   Each :py:class:`session.Session` is composed of a system load that will be
    executed on one or more Android devices.
-   A (subclass of) :py:class:`loads.SystemLoad` defines potential assets to
    deploy to the target Android devices before the load can be executed; and
    ultimately builds the command to be executed on the targets.
-   A :py:class:`android_tools.Device` (from the `Android tools`_ library) is a
    direct representation of a physical Android device available to the host by
    the ADB protocol. It relies on the `Python ADB library`_ to communicate with
    the Android devices without the need for the `adb` server to be running.

Usage
-----

This module can be called directly (the interactive shell
:py:class:`shell.MaestroShell` is the main entry-point), or imported and started
with the :py:func:`cmdloop()` method as it is built on the Python
:py:class:`cmd.Cmd` command interpreter.

..  warning:: The `adb` server does not play nicely with *Maestro*.

    Make sure the `adb` server is not running in the background as it keeps the
    device USB channels busy. In doubt, kill it before starting *Maestro*:

    .. code-block:: shell

        $ adb kill-server

As a library::

    >>> import android_maestro
    >>> SHELL = android_maestro.MaestroShell()
    # Start the interactive shell session
    >>> SHELL.cmdloop()

Or, as a standalone Python module:

.. code-block:: shell

    $ python -m android_maestro

Or, as a standalone (Python) application:

.. code-block:: shell

    $ maestro

A typical session within *Maestro* looks like::

    maestro> run

.. _`Android tools`: https://code.fairphone.com/gerrit/gitweb?p=vendor/fairphone/tools.git;f=python-libs/android-tools;a=history;hb=HEAD
.. _`Python ADB library`: https://github.com/google/python-adb
"""  # noqa

from . import loads, session, shell

# Expose the public API directly within the module
# pylint: disable=invalid-name, C0103

MaestroShell = shell.MaestroShell

Monkey = loads.Monkey

Session = session.Session
